﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using Microsoft.Practices.Composite.Presentation.Regions;
using Microsoft.Practices.Composite.Regions;
using Infragistics.Windows.Ribbon;

namespace InfraNCal
{
    public class XamRibbonRegionAdapter : RegionAdapterBase<XamRibbon>
    {
        private XamRibbon xamRibbonRegionTarget = null;

        public XamRibbonRegionAdapter(IRegionBehaviorFactory regionBehaviorFactory)
            : base(regionBehaviorFactory)
        { }

        #region Overrides

        protected override void Adapt(IRegion region, XamRibbon regionTarget)
        {
            this.xamRibbonRegionTarget = regionTarget;

            if (this.xamRibbonRegionTarget.Tabs.Count > 0)
            {
                foreach (object tab in this.xamRibbonRegionTarget.Tabs)
                {
                    region.Add(tab);
                }

                this.xamRibbonRegionTarget.Tabs.Clear();
            }

            if (this.xamRibbonRegionTarget.ContextualTabGroups.Count > 0)
            {
                foreach (object tabGroup in this.xamRibbonRegionTarget.ContextualTabGroups)
                {
                    region.Add(tabGroup);
                }

                this.xamRibbonRegionTarget.ContextualTabGroups.Clear();
            }

            foreach (object view in region.Views)
            {
                AddViewToRegion(view, this.xamRibbonRegionTarget);
            }

            region.ActiveViews.CollectionChanged += (s, args) =>
                {
                    switch (args.Action)
                    {
                        case NotifyCollectionChangedAction.Add:
                            {
                                foreach (object view in args.NewItems)
                                {
                                    AddViewToRegion(view, this.xamRibbonRegionTarget);
                                }
                                break;
                            }
                        case NotifyCollectionChangedAction.Remove:
                            {
                                foreach (object view in args.OldItems)
                                {
                                    RemoveViewFromRegion(view, this.xamRibbonRegionTarget);
                                }
                                break;
                            }
                        default:
                            {
                                // Do nothing.
                                break;
                            }
                    }
                };
        }

        protected override IRegion CreateRegion()
        {
            return new AllActiveRegion();
        }

        #endregion Overrides

        #region Private

        private void AddViewToRegion(object view, XamRibbon xamRibbon)
        {
            RibbonTabItem ribbonTabItem = view as RibbonTabItem;
            if (ribbonTabItem != null)
            {
                xamRibbon.Tabs.Add(ribbonTabItem);
            }
            else
            {
                ContextualTabGroup contextualTabGroup = view as ContextualTabGroup;
                if (contextualTabGroup != null)
                {
                    xamRibbon.ContextualTabGroups.Add(contextualTabGroup);
                }
            }
        }

        private void RemoveViewFromRegion(object view, XamRibbon xamRibbon)
        {
            RibbonTabItem ribbonTabItem = view as RibbonTabItem;
            if (ribbonTabItem != null)
            {
                xamRibbon.Tabs.Remove(ribbonTabItem);
            }
            else
            {
                ContextualTabGroup contextualTabGroup = view as ContextualTabGroup;
                if (contextualTabGroup != null)
                {
                    xamRibbon.ContextualTabGroups.Remove(contextualTabGroup);
                }
            }
        }

        #endregion Private
    }
}
